/*
 * [Vika] team in ACM SIDMOD 2015 Programming Contest
 *
 * Authors    : PhuongNhung Ngo,  ( maxnhong85@gmail.com  )
 *              ThaiSon Nguyen    ( thaison.ngn@gmail.com ) 
 * From       : Free University of Bozen-Bolzano, Karlsruhe Institute of Technology 
 *
 * Version    : 2.0
 * Date       : 04/05/2015
 */

#include <iostream>
#include <map>
#include <unordered_map>
#include <vector>
#include <cstdint>
#include <cstring>
#include <cstdlib>
#include <fstream>
#include <algorithm>

#include <tbb/parallel_for.h>
#include <tbb/blocked_range.h>
#include <tbb/parallel_do.h>
#include <tbb/task_scheduler_init.h>

//---------------------------------------------------------------------------
using namespace std;
//---------------------------------------------------------------------------
// Utility functions
//---------------------------------------------------------------------------
struct RangeIndex {
   uint64_t key;
   uint32_t first;
   uint32_t last;
   uint32_t next;
   inline bool operator<(const RangeIndex& o) const {
      return key < o.key;
   }
};
struct KeyIndex {
   uint64_t value;
   uint32_t index;
   inline bool operator<(const KeyIndex& o) const {
      return value < o.value;
   }
};
struct RowIndex {
   uint64_t* tuple;
   uint64_t trans;
};
struct ValidationIndex {
   uint64_t id;
   uint32_t query; 
   char result;
};
// Wire protocol messages
//---------------------------------------------------------------------------
struct MessageHead {
   /// Message types
   enum Type : uint32_t { Done, DefineSchema, Transaction, ValidationQueries, Flush, Forget };
   /// Total message length, excluding this head
   uint32_t messageLen;
   /// The message type
   Type type;
};
//---------------------------------------------------------------------------
struct DefineSchema {
   /// Number of relations
   uint32_t relationCount;
   /// Column log_counts per relation, one log_count per relation. The first column is always the primary key
   uint32_t columnCounts[];
};
//---------------------------------------------------------------------------
struct Transaction {
   /// The transaction id. Monotonic increasing
   uint64_t transactionId;
   /// The operation log_counts
   uint32_t deleteCount,insertCount;
   /// A sequence of transaction operations. Deletes first, total deleteCount+insertCount operations
   char operations[];
};
//---------------------------------------------------------------------------
struct TransactionOperationDelete {
   /// The affected relation
   uint32_t relationId;
   /// The row log_count
   uint32_t rowCount;
   /// The deleted values, rowCount primary keyss
   uint64_t keys[];
};
//---------------------------------------------------------------------------
struct TransactionOperationInsert {
   /// The affected relation
   uint32_t relationId;
   /// The row log_count
   uint32_t rowCount;
   /// The inserted values, rowCount*relation[relationId].columnCount values
   uint64_t values[];
};
//---------------------------------------------------------------------------
struct ValidationQueries {
   /// The validation id. Monotonic increasing
   uint64_t validationId;
   /// The transaction range
   uint64_t from,to;
   /// The query log_count
   uint32_t queryCount;
   /// The queries
   char queries[];
};
//---------------------------------------------------------------------------
struct Query {
   /// A column description   
   struct Column {
      /// Support operations
      enum Op : uint32_t { Equal, NotEqual, Less, LessOrEqual, Greater, GreaterOrEqual };
      /// The column id
      uint32_t column;
      /// The operations
      Op op;
      /// The constant
      uint64_t value;
      /// For comparision
      inline bool operator<(const Column& o) const {
         return (column*6+op) < (o.column*6+o.op);
      }
   };   
   /// The relation
   uint32_t relationId;
   /// The number of bound columns
   uint32_t columnCount;
   /// The bindings
   Column columns[];
};
//---------------------------------------------------------------------------
struct Flush {
   /// All validations to this id (including) must be answered
   uint64_t validationId;
};
//---------------------------------------------------------------------------
struct Forget {
   /// Transactions older than that (including) will not be tested for
   uint64_t transactionId;
};
//---------------------------------------------------------------------------
//static ofstream log_file;

#define BLOCK_SHIFT 10
#define BLOCK_DIFF ((1 << BLOCK_SHIFT) - 1)
#define SAMPLE_SIZE 256
#define RANGE_SMALL 5
#define GROUP_SIZE 512

class BlockTable {
public:
   BlockTable(uint32_t columnCount) {
      this->columnCount = columnCount;
      rowSize = columnCount*sizeof(uint64_t);

      blocks.reserve(64);
      current = 0;

      row_pool.reserve(1 << BLOCK_SHIFT);      
      row_lookup.reserve(1 << BLOCK_SHIFT);
      
      row_index.reserve(1 << BLOCK_SHIFT);
      row_del.reserve(1 << BLOCK_SHIFT);
      
      column_index.resize(columnCount);
      for (uint32_t j=0; j<columnCount; j++)
         column_index[j].reserve(1 << BLOCK_SHIFT);
      
      trans_index.reserve(1 << BLOCK_SHIFT);
      indexStart = 0; 
      
      column_status.resize(columnCount, true);
      analyzeStatus = true;          
   }
   
   ~BlockTable() {
      for (auto& block:blocks) free(block);
   }
   
   inline const uint32_t& countColumns() {
      return columnCount;
   }
      
   inline uint64_t* rowAt(const uint32_t& row) {
      return blocks[row >> BLOCK_SHIFT] + (row & BLOCK_DIFF) * columnCount;
   }
   
   inline uint32_t nextRow() {
      uint32_t idx;
      if (!row_pool.empty()) {
         idx = row_pool.back();
         row_pool.pop_back();
      } else {
         if ((current & BLOCK_DIFF) == 0) {
            blocks.push_back((uint64_t*) malloc(rowSize << BLOCK_SHIFT));
         }
         idx = current++;
      }
      return idx;
   }
   
   void remove(const uint64_t* keys, const uint32_t rowCount) {
      for (unsigned i=0; i<rowCount; i++) {
         const uint64_t& key = keys[i];
         if (row_lookup.count(key)) {
            const uint32_t& row = row_lookup[key];
            row_index.push_back({rowAt(row), transId});
            row_del.push_back({transId, row});
            row_lookup.erase(key);
         }
      }
   }

   void insert(const uint64_t* values, const uint32_t rowCount) {
      uint32_t size = row_index.size();
      row_index.resize(size+rowCount);
      for (unsigned i=0; i<rowCount; i++) {
         const uint64_t* data = values+i*columnCount;
         const uint32_t row = nextRow();
         uint64_t* tuple = rowAt(row);
         memmove(tuple, data, rowSize);
         row_index[size].tuple = tuple;
         row_index[size++].trans = transId;
         row_lookup[data[0]] = row;
      }
   }
   
   void startTransaction(const uint64_t transId) {
      this->transId = transId;
      transStart = row_index.size();
   }
   
   inline void stopTransaction() {
      uint32_t stop = row_index.size();
      if (stop > transStart) {
         trans_index.push_back({transId, transStart, stop});
      }
      transStart = stop;
   }

   void groupTransaction() {
      uint32_t j = indexStart;
      while (j < trans_index.size()) {
         uint32_t i = j, size = 0;
         while (size<GROUP_SIZE && i<trans_index.size()) {
            size += trans_index[i].last-trans_index[i].first;
            i++;
         }
         for (uint32_t k=j; k<i; k++) {
            trans_index[k].first = trans_index[j].first;
            trans_index[k].last = trans_index[i-1].last;
            trans_index[k].next = i-k;
         }
         j = i;
      }
   }

   void analyze() {
      if (analyzeStatus && current>=SAMPLE_SIZE) {
         vector<uint64_t> max, min;
         max.resize(columnCount, 0);
         min.resize(columnCount, UINT64_MAX);
         for (uint32_t j=0; j<SAMPLE_SIZE; j++) {
            uint64_t* row = rowAt(j);
            for (uint32_t col=1; col<columnCount; col++) {
               max[col] = row[col] > max[col] ? row[col] : max[col];
               min[col] = row[col] < min[col] ? row[col] : min[col];
            }
         }
         for (uint32_t col=1; col<columnCount; col++) {
            if ((max[col]-min[col]) < SAMPLE_SIZE/2) column_status[col] = false;
         }
         analyzeStatus = false;
      }
   }
      
   inline void createIndex(const RangeIndex& idx, const uint32_t& col) {
      uint32_t index = 0;
      for (uint32_t j=idx.first; j<idx.last; j++) {
         column_index[col][j].value = row_index[j].tuple[col];
         column_index[col][j].index = index++;
      }
      sort(column_index[col].begin()+idx.first, column_index[col].begin()+idx.last);
   }
   
   inline void createBound(const RangeIndex& idx, const uint32_t& col) {
      uint64_t max = 0, min = UINT64_MAX;
      for (uint32_t j=idx.first; j<idx.last; j++) {
         uint64_t value = row_index[j].tuple[col];
         max = value > max ? value : max;
         min = value < min ? value : min;
      }
      column_index[col][idx.first].value = min;
      column_index[col][idx.last-1].value = max;
   }
      
   void createIndex(const uint32_t col) {
      if (indexStart == trans_index.size()) return;
      column_index[col].resize(row_index.size());
      for (uint32_t i=indexStart; i<trans_index.size(); i+=trans_index[i].next) {
         if (column_status[col]) {
            createIndex(trans_index[i], col);
         } else
            createBound(trans_index[i], col);
      }
   }
   
	void completeIndex() {
      indexStart = trans_index.size();
	}
	
   void cleanTransaction(const uint64_t transId) {
      RangeIndex to_range = {transId};
      auto from_trans = trans_index.begin();
      auto to_trans = upper_bound(trans_index.begin(), trans_index.end(), to_range);
      if (from_trans != to_trans) {
         uint32_t count = (to_trans==trans_index.end()) ? row_index.size() : to_trans->first;
         // Clean transactions & indexes
         row_index.erase(row_index.begin(), row_index.begin()+count);
         trans_index.erase(from_trans, to_trans);
         indexStart = trans_index.size();
         for (uint32_t i=0; i<columnCount; i++) {
            column_index[i].erase(column_index[i].begin(), column_index[i].begin()+count);
         }
         // Update transaction index
         for (unsigned i=0; i<trans_index.size(); i++) {
            trans_index[i].first -= count;
            trans_index[i].last -= count;
         }         
         // Add deleted rows to the pool
         KeyIndex to_index = {transId};
         auto to_row = upper_bound(row_del.begin(), row_del.end(), to_index);
         for (auto row=row_del.begin(); row!=to_row; ++row) row_pool.push_back(row->index);
         row_del.erase(row_del.begin(), to_row);
      }
   }
   
   inline bool findRange(vector<KeyIndex>::iterator& from_row,
         vector<KeyIndex>::iterator& to_row, const Query::Column& c) {
      auto idx = from_row;
      KeyIndex p = {c.value};
      switch (c.op) {
         case Query::Column::Equal:
            idx = lower_bound(from_row, to_row, p);
            if (idx==to_row || idx->value!=c.value) return false;
            from_row = idx;
            while (idx!=to_row && idx->value==c.value) idx++;
            to_row = idx;
            break;
         case Query::Column::NotEqual:
            break;
         case Query::Column::Less:
            to_row = lower_bound(from_row, to_row, p); 
            break;
         case Query::Column::LessOrEqual:
            to_row = upper_bound(from_row, to_row, p);              
            break;
         case Query::Column::Greater:
            from_row = upper_bound(from_row, to_row, p);
            break;
         case Query::Column::GreaterOrEqual:
            from_row = lower_bound(from_row, to_row, p);
            break;
      }
      return from_row!=to_row;
   }
   
   inline bool findMinRange(const vector<RangeIndex>::iterator& iter, const Query& query,
         vector<KeyIndex>::iterator& from, vector<KeyIndex>::iterator& to) {
      uint32_t range = UINT32_MAX;
      for (uint32_t i=0; i<query.columnCount; i++) {
         auto& c = query.columns[i];
         if (!column_status[c.column]) continue;
         auto from_row = column_index[c.column].begin() + iter->first;
         auto to_row = column_index[c.column].begin() + iter->last;
         if (!findRange(from_row, to_row, c)) return false;
         uint32_t dt = distance(from_row, to_row);
         if (dt < range) {
            from = from_row; to = to_row;
            range = dt;
            if (range < RANGE_SMALL) return true;
         }
      }
      return from!=to;
   }
   
   inline bool validate(const uint64_t* tuple, const Query& query) {
      for (auto c=query.columns,cLimit=c+query.columnCount; c!=cLimit; ++c) {
         switch (c->op) {
            case Query::Column::Equal:
               if (tuple[c->column]!=c->value) return false;
               continue;
            case Query::Column::NotEqual:
               if (tuple[c->column]==c->value) return false;
               continue;
            case Query::Column::Less:
               if (tuple[c->column]>=c->value) return false;
               continue;
            case Query::Column::LessOrEqual:
               if (tuple[c->column]>c->value) return false;
               continue;
            case Query::Column::Greater:
               if (tuple[c->column]<=c->value) return false;
               continue;
            case Query::Column::GreaterOrEqual:
               if (tuple[c->column]<c->value) return false;
               continue;
         }
      }
      return true;
   }

   inline bool hasIndex(const Query& query) {
      Query::Column* col = (Query::Column*)query.columns;
      bool result = false;
      for (uint32_t j=0; j<query.columnCount; j++) {
         if (column_status[col[j].column] && col[j].op==0) {
            swap(col[j], col[0]); return true;
         }
         if (column_status[col[j].column]) result = true;
      }
      return result;
   }
   
   inline bool validateBound(const vector<RangeIndex>::iterator& iter, const Query& query) {
      for (uint32_t i=0; i<query.columnCount; i++) {
         auto& c = query.columns[i];
         auto min = column_index[c.column][iter->first].value;
         auto max = column_index[c.column][iter->last-1].value;
         bool result = false;
         switch (c.op) {
            case Query::Column::Equal:
               result=(c.value>=min && c.value<=max); break;
            case Query::Column::NotEqual:
               result=true; break;
            case Query::Column::Less:
               result=(min<c.value); break;
            case Query::Column::LessOrEqual:
               result=(min<=c.value); break;
            case Query::Column::Greater:
               result=(max>c.value); break;
            case Query::Column::GreaterOrEqual:
               result=(max>=c.value); break;
         }
         if (!result) return false;
      }
      return true;
   }
   
   bool validateTransaction(uint64_t from, uint64_t to, const Query& query) {
      RangeIndex from_range = {from};
      auto from_trans = lower_bound(trans_index.begin(), trans_index.end(), from_range);
      if (query.columnCount == 0) {
         RangeIndex to_range = {to};
         auto to_trans = upper_bound(from_trans, trans_index.end(), to_range);
         return from_trans!=to_trans;
      }

      bool useIndex = hasIndex(query);
      for (auto iter=from_trans; iter!=trans_index.end(); iter+=iter->next) {
         if (iter->key > to) break;
         if (useIndex) {
            vector<KeyIndex>::iterator from_row, to_row;
            if (!findMinRange(iter, query, from_row, to_row)) continue;
            if (distance(from_row,to_row)>RANGE_SMALL && !validateBound(iter, query)) continue;
            for (auto idx=from_row; idx!=to_row; idx++) {
               const RowIndex& row = row_index[iter->first+idx->index];
               if (row.trans<from || row.trans>to) continue;
               if (validate(row.tuple, query)) return true;
            }
         } else {
            if (!validateBound(iter, query)) continue;
            for (auto index=iter->first; index<iter->last; index++) {
               const RowIndex& row = row_index[index];
               if (row.trans<from || row.trans>to) continue;
               if (validate(row.tuple, query)) return true;
            }
         }
      }
      return false;
   }

private:   
   vector<uint64_t*> blocks;
   vector<vector<KeyIndex>> column_index;   
   unordered_map<uint64_t, uint32_t> row_lookup;
   vector<uint32_t> row_pool;
   vector<RowIndex> row_index;
   vector<KeyIndex> row_del;
   vector<RangeIndex> trans_index;

   uint32_t rowSize;
   uint32_t columnCount;
   uint32_t current;
   uint32_t indexStart;
   uint32_t transStart;
   uint64_t transId;
   
   vector<bool> column_status;   
   bool analyzeStatus;   
};

//---------------------------------------------------------------------------
// Begin solver implementation
//---------------------------------------------------------------------------

static vector<BlockTable> tables;

//---------------------------------------------------------------------------
inline void processDefineSchema(const DefineSchema& d) {
   tables.clear();
   for (unsigned i=0; i<d.relationCount; i++) {
      tables.push_back(BlockTable(d.columnCounts[i]));
   }
}
//---------------------------------------------------------------------------
inline void processTransaction(const Transaction& t, const unsigned first, const unsigned last) {
   const char* reader = t.operations;
   
   for (unsigned i=first; i<last; i++) tables[i].startTransaction(t.transactionId);
   
   for (unsigned i=0; i<t.deleteCount; ++i) {
      auto& o = *reinterpret_cast<const TransactionOperationDelete*>(reader);
      if (o.relationId>=first && o.relationId<last)
         tables[o.relationId].remove(o.keys, o.rowCount);
      reader += sizeof(TransactionOperationDelete) + (sizeof(uint64_t)*o.rowCount);
   }

   for (unsigned i=0; i<t.insertCount; ++i) {
      auto& o = *reinterpret_cast<const TransactionOperationInsert*>(reader);
      if (o.relationId>=first && o.relationId<last)
         tables[o.relationId].insert(o.values, o.rowCount);
      reader += sizeof(TransactionOperationInsert)+
         (sizeof(uint64_t)*o.rowCount*tables[o.relationId].countColumns());
   }

   for (unsigned i=first; i<last; i++) tables[i].stopTransaction();
}
//---------------------------------------------------------------------------
inline bool validateQuery(const Query& query) {
   Query::Column* col = (Query::Column*)query.columns;
   bool result = false;
   for (unsigned j=1; j<query.columnCount; j++) {
      unsigned i = j - 1;   
      if (col[j].column == col[i].column) {
         switch (col[i].op) {
            case Query::Column::Equal:   
               result = (col[j].op==0 && col[j].value!=col[i].value) ||
                  (col[j].op==1 && col[j].value==col[i].value) ||
                  (col[j].op==2 && col[j].value<=col[i].value) ||
                  (col[j].op==3 && col[j].value<col[i].value) ||
                  (col[j].op==4 && col[j].value>=col[i].value) ||
                  (col[j].op==5 && col[j].value>col[i].value);
               break;
            case Query::Column::NotEqual:
               result = (col[j].op==0 && col[j].value==col[i].value);
               break;
            case Query::Column::Less:
               result = (col[j].op==0 && col[j].value>=col[i].value) ||
                  (col[j].op==4 && col[j].value>=col[i].value) ||
                  (col[j].op==5 && col[j].value>col[i].value);
               break;
            case Query::Column::LessOrEqual:
               result = (col[j].op==0 && col[j].value>col[i].value) ||
                  (col[j].op>=4 && col[j].value>col[i].value);
               break;
            case Query::Column::Greater:
               result = (col[j].op==0 && col[j].value<=col[i].value) ||
                  (col[j].op==2 && col[j].value<=col[i].value) ||
                  (col[j].op==3 && col[j].value<col[i].value);               
               break;
            case Query::Column::GreaterOrEqual:
               result = (col[j].op==0 && col[j].value<=col[i].value) ||
                  (col[j].op==2 && col[j].value<col[i].value) ||
                  (col[j].op==3 && col[j].value<col[i].value);               
               break;            
         }
      }
      if (result) break;
   }
   return !result;
}
inline void processValidation(ValidationIndex& vi, char* query_buffer) {
   const ValidationQueries& v = *reinterpret_cast<const ValidationQueries*>(query_buffer+vi.query);
   bool conflict = false;
   const char* reader = v.queries;

   for (uint32_t i=0; i!=v.queryCount; ++i) {
      auto& q=*reinterpret_cast<const Query*>(reader);
      //Query::Column* col = (Query::Column*)q.columns;
      //sort(col, col+q.columnCount);      
      //if (validateQuery(q)) {
         conflict = tables[q.relationId].validateTransaction(v.from, v.to, q);
         if (conflict) break;
      //}
      reader+=sizeof(Query)+(sizeof(Query::Column)*q.columnCount);
   }

   vi.result = '0' + conflict;
}
//---------------------------------------------------------------------------
template<typename Type> static const Type& readBody(istream& in, vector<char>& buffer, uint32_t len) {
   buffer.reserve(len);
   in.read(buffer.data(), len);
   return *reinterpret_cast<const Type*>(buffer.data());
}
//---------------------------------------------------------------------------
#define THREAD_NUM 16
class QueryPool {
public:
   QueryPool() {
      query_max = 1024*1024*256; // Initial maximal size of query buffer
      query_buffer = (char*)malloc(query_max);
      query_index = 0;
      query_queue.reserve(1024*256);
      query_count = 0;
      
      trans_max = 1024*1024*32; // Initial maximal size of transaction buffer
      trans_buffer = (char*)malloc(trans_max);
      trans_queue.reserve(1024*32);
      trans_index = 0;
      
      lastForgetId = 0;
      
      tbb::task_scheduler_init init(THREAD_NUM);
   }
   ~QueryPool() {
      free(query_buffer);
      free(trans_buffer);
   }
   
   void executeQuery() {
      if (query_count == query_queue.size()) return;
      tbb::parallel_do(query_queue.begin()+query_count, query_queue.end(),
         [this] (ValidationIndex& idx) { processValidation(idx, query_buffer); });
      query_count = query_queue.size();
      query_index = 0;
   }

   void insertQuery(istream& in, uint32_t len) {
      if (query_index+len > query_max) processFlush();
      if (len > query_max) {
         free(query_buffer);
         while (query_max < len) query_max <<= 1;
         query_buffer = (char*)malloc(query_max);
      }
      char* current = query_buffer + query_index;
      in.read(current, len);

      const ValidationQueries& v = *reinterpret_cast<const ValidationQueries*>(current);
      query_queue.push_back({v.validationId, query_index, false});
      query_index += len;
   }
   
   void executeTransaction() {
      if (trans_queue.empty()) return;
      tbb::parallel_for(tbb::blocked_range<uint32_t>(0, tables.size()),
         [this] (tbb::blocked_range<uint32_t>& range) {
            for (uint32_t idx : trans_queue) {
               const Transaction& t = *reinterpret_cast<const Transaction*>(trans_buffer+idx);
               processTransaction(t, range.begin(), range.end());
            }
         });
      for (uint32_t i=0; i<tables.size(); i++) {
         tables[i].analyze();
         tables[i].groupTransaction();
      }
      tbb::parallel_for(tbb::blocked_range<uint32_t>(0, THREAD_NUM),
         [this] (tbb::blocked_range<uint32_t>& range) {
				for (uint32_t i=0; i<tables.size(); i++) {
					for (uint32_t col=range.begin(); col<tables[i].countColumns(); col+=THREAD_NUM)
						tables[i].createIndex(col);
				}
         });
      for (uint32_t i=0; i<tables.size(); i++) tables[i].completeIndex();
      trans_index = 0;
      trans_queue.clear();
   }
   
   void insertTransaction(istream& in, uint32_t len) {
      if (trans_index+len > trans_max) executeTransaction();
      if (len > trans_max) {
         free(trans_buffer);
         while (trans_max < len) trans_max <<= 1;
         trans_buffer = (char*)malloc(trans_max);
      }
      char* current = trans_buffer + trans_index;
      in.read(current, len);
      trans_queue.push_back(trans_index);
      trans_index += len;
   }
   
   void executeForget() {
      if (lastForgetId > 0) {
         tbb::parallel_do(tables.begin(), tables.end(),
            [this] (BlockTable& table) { table.cleanTransaction(lastForgetId); });
         lastForgetId = 0;
      }
   }
   
   void forgetTransaction(const Forget& f) {
      lastForgetId = f.transactionId;
   }
   
   void processFlush() {
      executeTransaction();
      executeQuery();
      executeForget();
   }
   
   void flushResult(const Flush& f) {
      if (query_queue.empty()) return;
      processFlush();
      unsigned i = 0;
      while (i<query_queue.size() && query_queue[i].id<=f.validationId) {
         cout.write(&query_queue[i].result, 1);
         i++;
      }
      cout.flush();
      query_queue.erase(query_queue.begin(), query_queue.begin()+i);
      query_count = query_queue.size();
   }
private:
   char* query_buffer;
   uint32_t query_index;
   uint32_t query_max;
   vector<ValidationIndex> query_queue;
   uint32_t query_count;

   char* trans_buffer;
   vector<uint32_t> trans_queue;
   uint32_t trans_index;
   uint32_t trans_max;
   
   uint64_t lastForgetId;
};
//---------------------------------------------------------------------------
int main()
{
   QueryPool queryPool;
   vector<char> message;
   message.reserve(1024);
   //log_file.open("log_file.txt");
   MessageHead head;
   while (true) {
      cin.read(reinterpret_cast<char*>(&head),sizeof(head));
      if (!cin) { cerr << "read error" << endl; abort(); }
      switch (head.type) {
         case MessageHead::Done:
		 	   return 0;
         case MessageHead::DefineSchema:
            processDefineSchema(readBody<DefineSchema>(cin,message,head.messageLen));
            break;
         case MessageHead::Transaction:
            queryPool.insertTransaction(cin, head.messageLen); break;
         case MessageHead::ValidationQueries:
            queryPool.insertQuery(cin, head.messageLen); break;
         case MessageHead::Flush:
            queryPool.flushResult(readBody<Flush>(cin,message,head.messageLen));break;
         case MessageHead::Forget:
            queryPool.forgetTransaction(readBody<Forget>(cin,message,head.messageLen)); break;
         default: cerr << "malformed message" << endl; abort();
      }
   }
   //log_file.close();
}
//---------------------------------------------------------------------------
